@@ -27,13 +27,14 @@
 """Classes for managing Http Cache Storage."""
 
 # Standard Library
-import datetime
 import logging
 import re
 
 from typing import Any, Optional, cast
 
 # 3rd Party Library
+from arrow import Arrow
+from rwskit.sqlalchemy.engine import SyncAlchemyEngine
 from scrapy.http.headers import Headers
 from scrapy.http.request import Request
 from scrapy.http.response import Response
@@ -43,17 +44,17 @@ from scrapy.settings import Settings
 from scrapy.spiders import Spider
 from scrapy.utils.misc import load_object
 from scrapy.utils.python import to_bytes, to_unicode
-from scrapy.utils.request import RequestFingerprinter
+from scrapy.utils.request import RequestFingerprinterProtocol
 from sqlalchemy import Engine
 from w3lib.http import headers_dict_to_raw, headers_raw_to_dict
 
 # 1st Party Library
 from scrachy.addons import try_import
 from scrachy.content import ContentExtractor
-from scrachy.db.engine import initialize_engine, session_scope
+from scrachy.db.engine import initialize_engine
 from scrachy.db.models import Response as CachedResponse
 from scrachy.db.models import ScrapeHistory
-from scrachy.db.repositories import ResponseRepository, ScrapeHistoryRepository
+from scrachy.db.repositories import SyncResponseRepository, SyncScrapeHistoryRepository
 from scrachy.exceptions import InvalidSettingError
 from scrachy.settings import iter_default_settings
 from scrachy.settings.defaults import filter as default_filter_settings
@@ -139,12 +140,12 @@ class AlchemyCacheStorage:
 
         # The following are created if necessary and set in the `open_spider` method
         # The callable used to fingerprint requests
-        self._fingerprinter: Optional[RequestFingerprinter] = None
+        self._fingerprinter: Optional[RequestFingerprinterProtocol] = None
 
         # The SqlAlchemy engine used to persist the responses. This is
         # either passed in as an existing engine or created from the
         # settings in `open_spider`.
-        # self._engine: Optional[Engine] = None
+        self._engine: Optional[SyncAlchemyEngine] = None
 
         # If we create the engine ourselves we should dispose of it when
         # the spider closes. Otherwise, let the creator deal with it.
@@ -166,6 +167,22 @@ class AlchemyCacheStorage:
     #     """
     #     return self._settings.get(f"SCRACHY_{name}".upper(), default)
 
+    @property
+    def fingerprinter(self) -> RequestFingerprinterProtocol:
+        """Return the request fingerprinter or raise an exception if it is not available."""
+        if self._fingerprinter is None:
+            raise ValueError("The fingerprinter has not been initialized.")
+
+        return self._fingerprinter
+
+    @property
+    def engine(self) -> SyncAlchemyEngine:
+        """Return the alchemy engine or raise an exception if it is not available."""
+        if self._engine is None:
+            raise ValueError("The engine is not available.")
+
+        return self._engine
+
     @property
     def is_scrapy_fingerprinter(self) -> bool:
         """
@@ -356,7 +373,7 @@ class AlchemyCacheStorage:
         if self._fingerprinter is None:
             raise ValueError("The request fingerprinter has not been initialized")
 
-        initialize_engine(spider.settings)
+        self._engine = initialize_engine(spider.settings)
 
         self.validate_settings()
 
@@ -405,10 +422,10 @@ class AlchemyCacheStorage:
             or response_retrieval_method == "full"
         ):
             if cached_response.meta:
-                meta_kwargs |= cached_response.meta  # type: ignore
+                meta_kwargs |= cached_response.meta
 
             if cached_response.headers:
-                raw_headers = headers_raw_to_dict(to_bytes(cached_response.headers))  # noqa
+                raw_headers = headers_raw_to_dict(to_bytes(cached_response.headers))
                 response_kwargs["headers"] = Headers(raw_headers)
 
             response_kwargs["status"] = cached_response.status
@@ -435,35 +452,25 @@ class AlchemyCacheStorage:
         :param response: The response to be stored in the cache as created by
                Scrapy's standard downloading process.
         """
-        # if not isinstance(response, TextResponse):
-        #     log.warning(f"The cache only supports TextResponses but received a '{type(response)}'")
-        #     return
-
-        # log.debug(f"response.body:\n{response.body}")
-        # response = cast(TextResponse, response)
-
-        # The fingerprinter is set in `open_spider`. We'll ignore the type
-        # checking error to avoid having to check if it's null every time we
-        # call this method.
-        fingerprint: bytes = self._fingerprinter.fingerprint(request)  # type: ignore
+        fingerprint: bytes = self.fingerprinter.fingerprint(request)
         timestamp = now_tzaware()
         cached_response = self._make_cached_response(
             fingerprint, timestamp, request, response
         )
 
-        # with self.session_scope() as session:
-        with session_scope() as session:
-            response_repo = ResponseRepository(session)
-            response_repo.upsert(cached_response)
+        with self.engine.session_scope() as session:
+            response_repo = SyncResponseRepository(self.engine)
+            response_repo.upsert(cached_response, session=session)
 
             if self.save_history:
-                history_repo = ScrapeHistoryRepository(session)
+                history_repo = SyncScrapeHistoryRepository(self.engine)
                 history_repo.insert(
                     ScrapeHistory(
                         fingerprint=fingerprint,
                         scrape_timestamp=timestamp,
                         body=to_bytes(response.text),
-                    )
+                    ),
+                    session=session,
                 )
 
     # endregion Scrapy API
@@ -491,9 +498,8 @@ class AlchemyCacheStorage:
         self._validate_supported_options()
         self._validate_database_parameters()
 
-    @staticmethod
-    def clear_cache():
-        with session_scope() as session:
+    def clear_cache(self):
+        with self.engine.session_scope() as session:
             session.query(Response).delete()
             session.query(Request).delete()
 
@@ -505,8 +511,7 @@ class AlchemyCacheStorage:
         :return: A list of SQLAlchemy result objects that contains all
                  the items in the cache.
         """
-        with session_scope() as session:
-            return list(ResponseRepository(session).find_all())
+        return list(SyncResponseRepository(self.engine).find_all_models())
 
     # endregion Extended API
     # endregion API
@@ -596,11 +601,9 @@ class AlchemyCacheStorage:
         # that is difficult to maintain.
         fingerprint = self._fingerprinter.fingerprint(request)  # type: ignore
 
-        log.debug(f"Looking for data with fingerprint: {fingerprint.hex()}")
-
         # with self.session_scope() as session:
-        with session_scope() as session:
-            repo = ResponseRepository(session)
+        with self.engine.session_scope() as session:
+            repo = SyncResponseRepository(self.engine)
             scrape_timestamp = repo.find_timestamp_by_fingerprint(fingerprint)
 
             # A missing timestamp indicates the data is not in the cache.
@@ -637,7 +640,7 @@ class AlchemyCacheStorage:
     def _make_cached_response(
         self,
         fingerprint: bytes,
-        timestamp: datetime.datetime,
+        timestamp: Arrow,
         request: Request,
         response: Response,
     ):
@@ -684,6 +687,7 @@ class AlchemyCacheStorage:
             response = response_cls(**response_kwargs)
 
         if response.request is not None:
+            log.debug(f"updating meta kwargs: {meta_kwargs}")
             response.meta.update(meta_kwargs)
 
         return response
