@@ -32,6 +32,7 @@ import queue
 import signal
 import subprocess
 import sys
+import time
 
 from struct import pack, unpack
 from subprocess import CalledProcessError
@@ -149,9 +150,6 @@ class SeleniumMiddleware:
         except IgnoreRequest as e:
             raise e
         except WebDriverException as e:
-            # Standard Library
-            import time
-
             request.retries += 1
             if request.retries > request.max_retries:
                 log.error(
@@ -180,8 +178,25 @@ class SeleniumMiddleware:
 
             return request
         except Exception as e:
-            log.error(f"Error processing request: {e}")
+            # Sometimes the client will hang when we catch this exception.
+            # I suspect it is because the shutdown/clean up code is done
+            # asynchronously. So, while the driver is still being cleaned up
+            # we have already initialized the new driver and changed its
+            # reference.
+            # I am not sure how to solve this problem correctly. For now it
+            # seems like adding a short delay between cleaning up
+            # (i.e., 'spider_closed') and reinitializing might be a
+            # semi-workable solution. A more robust solution, that would
+            # require a major refactor and introduce significant complexity
+            # (and additional surface for bugs) could be running the
+            # driver in a subprocess.
+            log.error(f"'{type(e)}': processing request: {e}")
             self.spider_closed()
+
+            # Sleep for 5 seconds before reinitializing the driver.
+            time.sleep(5.0)
+
+            # Reinitialize the driver.
             self._driver = self._initialize_driver()
 
             raise e
