@@ -17,20 +17,16 @@
 
 """Datetime utilities."""
 
-# Python Modules
-import datetime
-
-# 3rd Party Modules
-
-# Project Modules
+# 3rd Party Library
+import arrow
 
 
-def now_tzaware() -> datetime.datetime:
+def now_tzaware() -> arrow.Arrow:
     """Get a timezone aware ``datetime`` of the current instance and return it in the local timezone.
 
     Returns
     -------
-    datetime.datetime
+    Arrow
         The current instant.
     """
-    return datetime.datetime.now(datetime.timezone.utc).astimezone()
+    return arrow.utcnow().to("local")
